import xml.etree.ElementTree as ET

import os

from rtl_manifest import read_sources

TAG="{http://www.efinixinc.com/enf_proj}"


def create_project(cfg, filename):
    script_dir = os.path.dirname(__file__)

    # print(cfg)

    ET.register_namespace("efx", "http://www.efinixinc.com/enf_proj")
    tree = ET.parse(os.path.join(script_dir, "template.xml"))
    root = tree.getroot()

    # Step 0: Add Project info
    root.set("sw_version", cfg["prj_info"]["sw_version"])
    root.set("name", cfg["prj_info"]["name"])

    # Step 1: Add Device Info
    device_info = root.find(f"{TAG}device_info")
    assert device_info is not None

    for attribute in ["family", "device", "timing_model"]:
        element = ET.Element(f"efx:{attribute}", {"name": cfg["device_info"][attribute]})
        device_info.append(element)

    # Step 2: Add Design Info

    # Part 1: Add attributes
    design_info = root.find(f"{TAG}design_info")
    assert design_info is not None
    design_info.set("def_veri_version", cfg["design_info"]["verilog_version"])
    # ignore vhdl :)
    design_info.set("unified_flow", str(cfg["design_info"]["unified_flow"]).lower())    # hate yaml >:(

    # Part 2: Add elements

    top_module = ET.Element("efx:top_module", {"name": cfg["design_info"]["top_module"]})
    design_info.append(top_module)

    design_sources = read_sources(cfg["design_info"]["sources"])
    for source in design_sources:
        element = ET.Element("efx:design_file", {
            "name": source,
            "version": "default",
            "library": "default"
        })
        design_info.append(element)

    # Step 3: Add Constraints

    constraint_info = root.find(f"{TAG}constraint_info")
    assert constraint_info is not None

    sdc_element = ET.Element("efx:sdc_file", {"name": cfg["constraint_info"]["sdc_file"]})
    constraint_info.append(sdc_element)

    # Step 4: Add ISF

    isf_info = root.find(f"{TAG}isf_info")
    assert isf_info is not None

    isf_element = ET.Element("efx:isf_file", {"name": cfg["isf_info"]["isf_file"]})
    isf_info.append(isf_element)

    # Step 5: Add Synthesis Options (Skipping options for now)

    synthesis_options = root.find(f"{TAG}synthesis")
    assert synthesis_options is not None

    for macro_key, macro_value in cfg["synthesis_options"]["macros"].items():
        expanded_value = os.path.expandvars(macro_value)
        element = ET.Element("efx:defmacro", {"name": macro_key, "value": expanded_value})
        synthesis_options.append(element)


    ET.indent(tree, '    ')
    tree.write(f"{filename}", encoding="utf-8", xml_declaration=False)

def build_project(cfg):
    name = f"{cfg['prj_info']['name']}.xml"
    create_project(cfg, name)
    os.system(f"efx_run --prj {name} --output_dir outflow --work_dir work")

