import rtl_manifest
import os

def build_nonprj(cfg):
    all_sources = rtl_manifest.read_sources(cfg["design_info"]["sources"])

    verilog_sources = list(filter(lambda s: (s.endswith(".v") or s.endswith(".sv")), all_sources))
    xci_sources = list(filter(lambda s: s.endswith(".xci"), all_sources))
    xdc_sources = list(filter(lambda s: s.endswith(".xdc"), all_sources))
    tcl_sources = list(filter(lambda s: s.endswith(".tcl"), all_sources))

    top = cfg["design_info"]["top_module"]
    part = cfg["device_info"]["device"]

    synth_directive = "default"
    opt_directive = "default"
    place_directive = "default"
    route_directive = "default"

    if "synthesis_options" in cfg.keys():
        if "synth_directive" in cfg["synthesis_options"]:
            synth_directive = cfg["synthesis_options"]["synth_directive"]
        if "opt_directive" in cfg["synthesis_options"]:
            opt_directive = cfg["synthesis_options"]["opt_directive"]

    if "pnr_options" in cfg.keys():
        if "place_directive" in cfg["pnr_options"]:
            place_directive = cfg["pnr_options"]["place_directive"]
        if "route_directive" in cfg["pnr_options"]:
            route_directive = cfg["pnr_options"]["route_directive"]

    with open("build.tcl", "w") as f:
        f.write(f"set outputDir ./nonprojectflow\n")
        f.write(f"file mkdir $outputDir\n")

        f.write(f"set_part {part}\n")

        f.write(f"read_verilog -sv {' '.join(verilog_sources)}\n")
        f.write(f"read_ip \"{' '.join(xci_sources)}\"\n")
        f.write(f"read_xdc \"{' '.join(xdc_sources)}\"\n")
        
        f.write(f"synth_ip [get_ips *]\n")
        f.write(f"synth_design -top {top} -directive {synth_directive}\n")
        f.write(f"write_checkpoint -force $outputDir/post_synth.dcp\n")
        f.write(f"report_utilization -hierarchical -file $outputDir/post_synth_util.rpt\n")

        if tcl_sources is not []:
            for tcl_script in tcl_sources:
                f.write(f"source {tcl_script}\n")

        f.write(f"opt_design -directive {opt_directive}\n")
        f.write(f"place_design -directive {place_directive}\n")
        f.write(f"write_checkpoint -force $outputDir/post_place.dcp\n")
        f.write(f"route_design -directive {route_directive}\n")
        f.write(f"write_checkpoint -force $outputDir/post_route.dcp\n")
        f.write(f"report_utilization -file $outputDir/post_impl_util.rpt\n")
        f.write(f"report_timing -file $outputDir/timing.rpt\n")
        f.write(f"report_power -file $outputDir/power.rpt\n")
        f.write(f"write_bitstream -force $outputDir/{top}.bit\n")
        f.write(f"exit\n")

    os.system(f"vivado -nolog -nojournal -mode tcl -source build.tcl")
    os.remove("build.tcl")
